#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Note: To use the 'upload' functionality of this file, you must:
#   $ pip install twine
#
# Commands:
# ### build source distribute:
#
# python setup.py dist
#
# ### publish package to PyPi
#
# python setup.py publish
#
# Note : make sure the .pypirc file is created in your home directory:
# #############################
#
# [distutils]
# index-servers =
#          pypi
#
# [pypi]
# repository = https://upload.pypi.org/legacy/
# username = <pypi_username>
# password = <pypi_password>
#
##################################
#
#

import io
import os
import sys
from shutil import rmtree

from setuptools import find_packages, setup, Command

# Package meta-data.
NAME = 'merc-common'
VERSION = '1.0.0'
DESCRIPTION = 'CRAMS API opensource merc-common package.'
URL = 'https://github.com/CRAMS-Dashboard/crams-api'
AUTHOR = 'Monash University e-Research Centre'
EMAIL = 'crams@monash.edu'

REQUIRES_PYTHON = '>=3.8.0'

# What packages are required for this module to be executed?
REQUIRED = [
    'Django>=3.2',
    'djangorestframework>=3.12',
    'django-rest-auth>=0.9.5',
    'django-cors-headers>=3.7.0',
    'django-extensions>=3.1.3',
    'django-filter>=2.4.0',
    'django-model-utils>=4.1.1',
    'mysqlclient>=2.0.3',
    'rest_condition>=1.0.3',
    'gunicorn>=20.0.4',
    'urllib3>=1.26.5',
    'sqlparse>=0.4.1',
    'pandas>=1.2.3',
    'PyJWT>=1.7.1',
]

# What packages are optional?
EXTRAS = {
    # 'fancy feature': ['django'],
}

# get the current directory
current_dir = os.path.abspath(os.path.dirname(__file__))

# Import the README and use it as the long-description.
# Note: this will only work if 'README.rst' is present in your MANIFEST.in file!
try:
    with io.open(os.path.join(current_dir, 'README.rst'), encoding='utf-8') as f:
        long_description = '\n' + f.read()
except FileNotFoundError:
    long_description = DESCRIPTION

# define the version number in __init__.py file
#  eg:
# __version__ = '1.0.0'
#
# Load the package's __init__.py module as a dictionary.
#
about = {}
# project_slug = NAME.lower().replace("-", "_").replace(" ", "_")
# with open(os.path.join(current_dir, project_slug, '__init__.py')) as vf:
#     exec(vf.read(), about)
with open(os.path.join(current_dir, 'version.py'))as fv:
    exec(fv.read(), about)

current_version = about.get('__version__', None)
if not current_version:
    about['__version__'] = VERSION


def status(s):
    """Prints things in bold."""
    print('\033[1m{0}\033[0m'.format(s))


class DistributeSource(Command):
    """ python setup.py dist."""

    description = 'Build the package.'
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            status('Removing previous build dir ...')
            rmtree(os.path.join(current_dir, 'build'))

            status('Removing previous dist dir ...')
            rmtree(os.path.join(current_dir, 'dist'))
        except OSError:
            pass
        status('Building Source and Wheel (universal) distribution ...')
        os.system('{0} setup.py sdist bdist_wheel --universal'.format(sys.executable))
        sys.exit()


class PublishDist(Command):
    """Support setup.py upload."""

    description = 'Publish the package.'
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        status('Uploading the package to PyPI via Twine ...')
        os.system('twine upload dist/*')
        #
        # status('Pushing git tags…')
        # os.system('git tag v{0}'.format(about['__version__']))
        # os.system('git push --tags')
        sys.exit()


# Where the magic happens:
setup(
    name=NAME,
    version=about['__version__'],
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type='text/markdown',
    author=AUTHOR,
    author_email=EMAIL,
    python_requires=REQUIRES_PYTHON,
    url=URL,
    packages=find_packages(exclude=["tests", "*.tests", "*.tests.*", "tests.*"]),

    # If your package is a single module, use this instead of 'packages':
    # py_modules=['mypackage'],

    # entry_points={
    #     'console_scripts': ['mycli=mymodule:cli'],
    # },

    install_requires=REQUIRED,
    extras_require=EXTRAS,
    include_package_data=True,
    license='GNU General Public License v3.0',
    classifiers=[
        # Full list: https://pypi.python.org/pypi?%3Aaction=list_classifiers
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Framework :: Django :: 3.2',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    # setup.py dist and publish
    cmdclass={
        'dist': DistributeSource,
        'publish': PublishDist,
    },
)



# Generated by Django 3.1.1 on 2020-09-25 02:22

from django.conf import settings
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('crams', '0001_auth'),
    ]

    operations = [
        migrations.CreateModel(
            name='EResearchBody',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(db_index=True, max_length=200)),
                ('email', models.EmailField(blank=True, max_length=254, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='EResearchBodySystem',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(db_index=True, max_length=200)),
                ('email', models.EmailField(blank=True, max_length=254, null=True)),
                ('e_research_body', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, related_name='systems', to='crams.eresearchbody')),
            ],
            options={
                'unique_together': {('name', 'e_research_body')},
                'index_together': {('name', 'e_research_body')},
            },
        ),
        migrations.CreateModel(
            name='EResearchBodyIDKey',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('type', models.CharField(choices=[('I', 'Id_Key'), ('L', 'Label'), ('M', 'Metadata')], default='I', max_length=1)),
                ('key', models.CharField(max_length=100)),
                ('description', models.TextField(blank=True, null=True)),
                ('e_research_body', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.DO_NOTHING, to='crams.eresearchbody')),
            ],
            options={
                'unique_together': {('key', 'e_research_body')},
            },
        ),
        migrations.CreateModel(
            name='EResearchBodyDelegate',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(db_index=True, max_length=200)),
                ('email', models.EmailField(blank=True, max_length=254, null=True)),
                ('e_research_body', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, related_name='delegates', to='crams.eresearchbody')),
            ],
            options={
                'unique_together': {('name', 'e_research_body')},
                'index_together': {('name', 'e_research_body')},
            },
        ),
    ]



# Generated by Django 3.1.1 on 2020-09-25 02:31

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('crams', '0004_provider'),
    ]

    operations = [
        migrations.CreateModel(
            name='SupportEmailContact',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('description', models.CharField(blank=True, max_length=255, null=True)),
                ('email', models.EmailField(db_index=True, max_length=254)),
                ('e_research_body', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, to='crams.eresearchbody')),
                ('e_research_system', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.DO_NOTHING, to='crams.eresearchbodysystem')),
            ],
        ),
        migrations.CreateModel(
            name='MessageOfTheDay',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('creation_ts', models.DateTimeField(auto_now_add=True)),
                ('last_modified_ts', models.DateTimeField(auto_now=True)),
                ('message', models.CharField(blank=True, max_length=1500, null=True)),
                ('created_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='messageoftheday_created_by', to=settings.AUTH_USER_MODEL)),
                ('e_research_body', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.DO_NOTHING, to='crams.eresearchbody')),
                ('updated_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='messageoftheday_updated_by', to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



# coding=utf-8
"""
Crams Models admin
"""

from django.contrib import admin
from django.contrib.auth import admin as auth_admin
from django.db import models as django_models
from django.forms import Textarea
from django.utils.translation import ugettext_lazy as _

from crams import models


# class UserAdmin(auth_admin.UserAdmin):
#     """
#     class UserAdmin
#     """
#     list_display = ('first_name', 'last_name', 'username',
#                     'email', 'is_staff', 'is_superuser', 'is_active')
#
#     list_filter = ['is_staff', 'is_superuser', 'is_active']


class ParentListFilter(admin.SimpleListFilter):
    # Human-readable title which will be displayed in the
    # right admin sidebar just above the filter options.
    title = _('parent')

    # Parameter for the filter that will be used in the URL query.
    parameter_name = 'current'

    def lookups(self, request, model_admin):
        """
        Returns a list of tuples. The first element in each
        tuple is the coded value for the option that will
        appear in the URL query. The second element is the
        human-readable name for the option that will appear
        in the right sidebar.
        """
        return (
            ('all', _('all')),
            (None, _('current only')),
        )

    def queryset(self, request, queryset):
        """
        Returns the filtered queryset based on the value
        provided in the query string and retrievable via
        `self.value()`.
        """
        # Compare the requested value (either '80s' or '90s')
        # to decide how to filter the queryset.
        if not self.value():
            if queryset.model.__name__ == 'Project':
                return queryset.filter(current_project__isnull=True)
            elif queryset.model.__name__ == 'Request':
                return queryset.filter(current_request__isnull=True)
        else:
            return queryset


class DomainAdmin(admin.ModelAdmin):
    """
     class DomainAdmin
    """
    list_display = ('id', 'percentage', 'project', 'for_code')
    list_filter = (ParentListFilter,)


class GrantAdmin(admin.ModelAdmin):
    """
     class GrantAdmin
    """
    list_display = ('id', 'grant_type', 'funding_body_and_scheme', 'start_year', 'duration', 'total_funding', 'project')
    list_filter = (ParentListFilter,)


class RequestAdmin(admin.ModelAdmin):
    """
    class RequestAdmin
    """
    list_display = ('id', 'project', 'funding_scheme', 'start_date',
                    'end_date', 'request_status', 'allocation_extension_count', 'current_request')
    list_filter = [ParentListFilter, 'e_research_system', 'current_request', 'allocation_extension_count']


class ProjectAdmin(admin.ModelAdmin):
    """
    class ProjectAdmin
    """
    list_display = ('id', 'title', 'description', 'current_project')
    list_filter = (ParentListFilter, 'requests__e_research_system')


class ProjectIDAdmin(admin.ModelAdmin):
    list_display = ('identifier', 'system', 'parent_erb_project_id',
                    'provision_details', 'project')
    list_filter = ['provision_details__status', 'system', 'project']


class EResearchBodyIDKeyAdmin(admin.ModelAdmin):
    list_display = ('key', 'e_research_body', 'type')
    list_filter = ['e_research_body', 'type']


class ContactDetailAdmin(admin.ModelAdmin):
    """
    class ContactAdmin
    """
    list_display = ('type', 'email', 'phone', 'parent_contact')


class ContactAdmin(admin.ModelAdmin):
    """
    class ContactAdmin
    """
    list_display = ('given_name', 'surname', 'email')


class ProjectContactAdmin(admin.ModelAdmin):
    """
    class ProjectContactAdmin
    """
    list_display = ('contact', 'contact_role', 'project')
    list_filter = ['contact', 'project', 'contact_role']


class PublicationAdmin(admin.ModelAdmin):
    """
    class ProjectContactAdmin
    """
    list_display = ('project', 'reference')
    list_filter = ['project']


class FundingSchemeAdmin(admin.ModelAdmin):
    """
    class FundingSchemeAdmin
    """
    list_display = ('id', 'funding_scheme', 'funding_body')


class FundingBodyAdmin(admin.ModelAdmin):
    """
    class FundingBodyAdmin
    """
    list_display = ('id', 'name')


class CramsTokenAdmin(admin.ModelAdmin):
    """
    class FundingBodyAdmin
    """
    list_display = ('user', 'key', 'ks_roles')


class EResearchBodySystemAdmin(admin.ModelAdmin):
    """
    class EResearchBodySystemAdmin
    """
    list_display = (
        'name',
        'e_research_body',
        'email'
    )


class ERBRequestStatusAdmin(admin.ModelAdmin):
    """
        class ERBRequestStatusAdmin
        """
    list_display = ('display_name', 'request_status',
                    'e_research_system', 'e_research_body', 'extension_count_data_point')
    list_filter = ['request_status', 'extension_count_data_point', 'e_research_body']


class EResearchBodyDelegateAdmin(admin.ModelAdmin):
    """
    class EResearchBodyDelegateAdmin
    """
    list_display = (
        'name',
        'e_research_body',
        'email'
    )


class EResearchContactIdentifierAdmin(admin.ModelAdmin):
    list_display = ('identifier', 'contact',
                    'system', 'parent_erb_contact_id', 'provision_details')
    list_filter = ['provision_details__status', 'system', 'contact']


class SupportEmailContactAdmin(admin.ModelAdmin):
    """
    class SupportEmailContact
    """
    list_display = ('description', 'email',
                    'e_research_body', 'e_research_system')
    list_filter = ['e_research_body', 'e_research_system']


class MessageOfTheDayAdmin(admin.ModelAdmin):
    """
    class MessageOfTheDayAdmin
    """
    exclude = ('created_by', 'updated_by')
    list_display = ('message', 'creation_ts', 'created_by', 'updated_by')
    list_filter = ['creation_ts', 'created_by', 'updated_by']

    formfield_overrides = {
        django_models.CharField: {'widget': Textarea(attrs={'rows': 6, 'cols': 60})},
    }

    def get_actions(self, request):
        # Disable delete
        actions = super(MessageOfTheDayAdmin, self).get_actions(request)
        if actions and 'delete_selected' in actions:
            del actions['delete_selected']
        return actions

    def has_delete_permission(self, request, obj=None):
        return False

    def save_model(self, request, obj, form, change):
        if getattr(obj, 'created_by', None) is None:
            # new message
            obj.created_by = request.user
            obj.updated_by = request.user
        else:
            # editing message
            obj.updated_by = request.user

        obj.save()

    def __init__(self, *args, **kwargs):
        super(MessageOfTheDayAdmin, self).__init__(*args, **kwargs)
        self.list_display_links = ()


admin.site.register(models.User, auth_admin.UserAdmin)
admin.site.register(models.FundingScheme, FundingSchemeAdmin)
admin.site.register(models.FundingBody, FundingBodyAdmin)
admin.site.register(models.Provider)
admin.site.register(models.CramsToken, CramsTokenAdmin)
admin.site.register(models.EResearchBodyIDKey, EResearchBodyIDKeyAdmin)
admin.site.register(models.Question)
admin.site.register(models.EResearchBody)
admin.site.register(models.EResearchBodySystem, EResearchBodySystemAdmin)
admin.site.register(models.EResearchBodyDelegate, EResearchBodyDelegateAdmin)
admin.site.register(models.SupportEmailContact, SupportEmailContactAdmin)
admin.site.register(models.MessageOfTheDay, MessageOfTheDayAdmin)



# coding=utf-8
"""

"""
from rest_framework import exceptions

from crams.utils import date_utils


def extract_http_query_params(param_validationfnList_dict, http_request_obj):
    ret_dict = dict()

    for param, fn_list in param_validationfnList_dict.items():
        if param in http_request_obj.query_params:
            value = http_request_obj.query_params.get(param)
            for fn in fn_list:
                value = fn(param, value)
            ret_dict[param] = value
    return ret_dict


def extract_validate_post_params(
        param_validationfnList_dict, http_request_obj):
    ret_dict = dict()
    post_data = http_request_obj.data
    for param, fn_list in param_validationfnList_dict.items():
        if param in post_data:
            value = post_data.get(param)
            for fn in fn_list:
                value = fn(param, value)
            ret_dict[param] = value
    return ret_dict


def is_required_param_function(param_name, value):
    if value is None:
        raise exceptions.ValidationError(
            '{} param is required'.format(param_name))
    return value


def is_param_numeric_function(param_name, value):
    if not value.isnumeric():
        raise exceptions.ValidationError(
            '{} param must be numeric'.format(param_name))
    return value


def is_param_date_function(param_name, value):
    try:
        date_ts_obj = date_utils.parse_datetime_from_str(value)
        return date_ts_obj.date()
    except ImportError:
        msg = '{}: date format incorrect'.format(param_name)
        raise exceptions.ValidationError(msg)



import pprint


class Debug:
    def __init__(self):
        self.pp = pprint.PrettyPrinter(indent=4)  # For Debug

    def pprint(self, obj):
        """print."""
        if not hasattr(self, 'pp'):
            pass
        self.pp.pprint(obj)
